from gtkmvc.controller import Controller
from gtk import main_quit, ListStore, CellRendererText, TreeViewColumn, \
                Statusbar, ProgressBar, Dialog, TreeRowReference, \
                SELECTION_MULTIPLE, DIALOG_MODAL
from birne_view import BirneSelectorView
from os import path, environ, stat
from time import sleep
import gobject


class BirneController (Controller):
    """
    """

    def __init__(self, model):
        """Contructor. model will be accessible via the member 'self.model'.
        Registration is also performed."""
        Controller.__init__(self, model)
        self.model.registerObserver(self)

        # This is the current working directory for the file selector.
        # We have to remember this because the selector is instantiated
        # on every click on the add button.
        self.dir = str(environ['HOME'])

        pass

    def registerView(self, view):
        Controller.registerView(self, view) # Calls the overloaded method

        # Connects the exiting signal:
        view.get_top_widget().connect("destroy", main_quit)
        view["quit"].connect("clicked", main_quit)

        #
        self.listStore = ListStore(gobject.TYPE_STRING, gobject.TYPE_INT)
        self.view["CDContent"].set_model(self.listStore)
        self.view["CDContent"].get_selection().set_mode(SELECTION_MULTIPLE)

        renderer = CellRendererText()
        nameColumn = TreeViewColumn("Groesse (kiB)", renderer, text=1)
        sizeColumn = TreeViewColumn("Datei", renderer, text=0)
        self.view["CDContent"].append_column(nameColumn)
        self.view["CDContent"].append_column(sizeColumn)
        pass

    # Signal (view -> self: hey, the user talks to us!)
    def on_add_clicked(self, button):
        self.model.status = "on_addFile_clicked %s" % button
        selectorCtrl = BirneSelectorController(self.model, self.dir)
        selector = BirneSelectorView(selectorCtrl)
        pass

    def on_remove_clicked(self, button):
        self.model.status = "on_remove_clicked %s" % button
        model, selection = self.view["CDContent"].get_selection().get_selected_rows()
        print selection
        for path in selection:
            print model.get_iter_from_string( str(path) )
        pass

    def on_clear_clicked(self, button):
        self.model.status = "on_remove_allFiles_clicked %s" % button
        self.model.content = {}
        self.model.contentsize = 0
        pass

    def on_burn_clicked(self, button):
        progressbar = ProgressBar()
        self.view["progressbar"].show()
        self.model.burn()
        #thread.start()
        #while thread.isAlive():
        #    #thread.join(0.1)
        #    sleep(0.5)
        #    if thread.progress:
        #        current, total = thread.progress
        #        progressbar.set_fraction( int(current) * 1.0 / int(total) )
        #        #progressbar.pulse()
        #dialog.destroy()
        pass

    def on_mode_toggled(self, button):
        self.model.mode = button.get_name()
        pass

    def on_speed_value_changed(self, button):
        self.model.speed = button.get_value()

    def on_eject_toggled(self, button):
        self.model.eject = button.get_active()
        pass

    def on_simulate_toggled(self, button):
        self.model.simulate = button.get_active()
        pass

###

    # Observable notifications (model -> self: hey, I am morphing!)
    def property_progress_change_notification(self, model, old, new):
        current, total = [int(s) for s in new]
        fraction = current * 1.0 / total
        if fraction < 1:
            self.view["progressbar"].set_fraction(fraction)
            self.view["progressbar"].set_text("%s / %s MB" % current, total)
        else:
            self.view["progressbar"].hide()

    def property_speed_change_notification(self, model, old, new):
        """  """
        self.model.status = "C: model property speed changed: %s" % new

        pass

    def property_eject_change_notification(self, model, old, new):
        """  """
        self.model.status = "C: model property eject toggled: %s" % new
        pass

    def property_simulate_change_notification(self, model, old, new):
        """  """
        self.model.status = "C: model property simulate toggled: %s" % new
        pass

    def property_content_change_notification(self, model, old, new):
        """ Rebuilds the whole CDContent TreeView. """
        self.model.status = "C: model property content changed: %s elements" % len(new)
        self.listStore.clear()
        sorted_targets = self.model.content.keys()
        sorted_targets.sort()
        # temporarily disconnect model from view to avoid unnecessary
        # updates
        self.view["CDContent"].set_model(None)
        for target in sorted_targets:
            abs_path = self.model.content[target]
            # os.stat(file) returns a list with file information with its size
            # in bytes at [6]
            size = stat(abs_path)[6] / 1024
            self.listStore.append( (target, size ) )
        # reconnect model
        self.view["CDContent"].set_model(self.listStore)
        pass

    def property_mode_change_notification(self, model, old, new):
        """  """
        self.model.status = "C: model property mode changed: %s" % new
        if new == "copy":
            self.view["contentButtonBox"].foreach( lambda w: w.set_sensitive(False) )
            self.view["contentScrolledWindow"].set_sensitive(False)
        elif old == "copy":
            self.view["contentButtonBox"].foreach( lambda w: w.set_sensitive(True) )
            self.view["contentScrolledWindow"].set_sensitive(True)
        pass

    def property_contentsize_change_notification(self, model, old, new):
        """  """
        displaysize = str ( new / 1024 / 1024 ) + " MiB"
        self.view["size"].set_label(displaysize)
        pass

    def property_status_change_notification(self, model, old, new):
        id = self.view["statusbar"].get_context_id("property")
        self.view["statusbar"].push(id, str(new))
        pass


    def property_discfree_change_notification(self, model, old, new):
        displaysize = str ( new / 1024 / 1024 ) + " MiB"
        self.view["discfree"].set_text(displaysize);
        pass

    pass # end of class

class BirneSelectorController(Controller):

    def __init__(self, model, dir=str(environ['HOME'])):
        """Contructor. model will be accessible via the member 'self.model'.
        Registration is also performed."""
        Controller.__init__(self, model)
        self.model.registerObserver(self)
        self.dir = dir
        pass

    def registerView(self, view):
        Controller.registerView(self, view) # Calls the overloaded method

        if path.isdir(self.dir):
            self.view["mainWindow"].set_current_folder(self.dir)
        pass

    def on_close_clicked(self, button):
        self.view.get_top_widget().destroy()
        pass

    def on_select_clicked(self, button):
        new_files = self.view["mainWindow"].get_filenames()
        strip_folder = self.view["mainWindow"].get_current_folder()
        is_relative = self.view["rel_paths"].get_active()
        self.model.add_files(new_files, is_relative, strip_folder)
        #print new_files
        pass

    def on_select_and_close_clicked(self, button):
        self.on_select_clicked(button)
        self.on_close_clicked(button)

    pass

class BirneDialogController(Controller):
    def __init__(slef, model):
        Controller.__init__(self, model, thread=None)
        self.model.registerObserver(self)
        pass

    def on_cancel_clicked(self, button):
        pass

# vim: expandtab ts=4
