from gtkmvc.model import Model
import os
import statvfs
import re
import gtk
from popen2 import Popen3
from select import select
from fcntl import fcntl, F_SETFL
from threading import Thread
from time import sleep


device = "/dev/hdc"
cdrecord = "/usr/bin/cdrecord "
mkisofs = "/usr/bin/mkisofs "
cdrdao = "/usr/bin/cdrdao "
tmpdir = "/tmp"

burnfree = True

def check_media_empty():
    """check if cdrom is empty using cdrdao"""
    # taken almost verbatim from Gaetano Paolone's burn v0.4.3
    # giuppi's function 
    media_raw_info=popen2.popen2('%s disk-info --device %s 2>/dev/null'%(cdrdao,device))
    lines=media_raw_info[0].readlines()
    for line in lines:
        if line.startswith('CD-R empty'):
            if line.split(':')[1].strip()=='yes':
                return True
            return False
            while gtk.events_pending():
                gtk.main_iteration()
    return False

def get_media_capacity():
    """get media capacity using cdrdao"""
    # taken almost verbatim from Gaetano Paolone's burn v0.4.3
    # giuppi's function 
    media_raw_info=popen2.popen2('%s disk-info --device %s 2>/dev/null'%(cdrdao,device))
    lines=media_raw_info[0].readlines()
    for line in lines:
        if line.startswith('Total Capacity'):
            if line.split(':')[1].strip()=='n/a':
                return None
            return line.split()[6].split('/')[0]
    return None

def get_target(absfile, is_relative, strip_path):
    """ Returns the target filename from an absolute source name. """
    relfile = absfile.split(strip_path, 1)[1]
    return ( is_relative and relfile ) or file



class BirneModel (Model):
    """ Model for Birne's main GUI. It contains everything necessary to
    build various types of CDs."""

    # observable properties:
    __properties__ = {
        "mode"          : "copy",    # type of disc
        "speed"         : 4,         # burning speed
        "simulate"      : True,     # burn in dummy mode?
        "eject"         : True,      # eject when finished?
        "content"       : {},        # content of disc. keys are target
                                     # paths, values are source paths
        "contentsize"   : 0,         # size of disc content
        "status"        : "",        # status message
        "progress"      : "",
        "discfree"      : 0          # free space on partition where tmpdir resides
        }


    def __init__(self):
        Model.__init__(self)
        pass

    def burn(self):
        """Executes the burning function appropriate for current mode."""
        method = getattr(self, "burn_%s" % self.mode, "burn_copy")
        return method()

    def burn_copy(self):
        self.status = "burn copy"
        pass

    def burn_audio(self):
        self.status = "burn audio"
        pass

    def burn_files(self):
        self.status = "burn files"
        pass

    def burn_image(self):
        self.status = "burn image"
        cdrecord_line = "%s -v -pad -tao speed=%s dev=%s driveropts=burnfree " \
                         % (cdrecord, self.speed, device)
        if self.simulate: cdrecord_line += "-dummy "
        if self.eject: cdrecord_line += "-eject "
        imagefile = " ".join( [self.content[key] for key in self.content] )
        cdrecord_line += "-data %s" % imagefile
        self.status = cdrecord_line
        self.run_cdrecord(cdrecord_line);
        #thread = Cdrecord(cdrecord_line)
        #thread.start()
        #return thread
        return

    def add_files(self, new_files, is_relative=True, strip_path=""):
        """Adds list of files and directories to CD content. Traverses through
        directories contained in new_files and adds every single file with its
        complete path. If is_relative is True, strip_path will be truncated
        from the path."""

        # Get a copy of current CD content...
        new_content = self.content.copy()

        # ... get new files...
        for file in new_files:
            # this check makes sure we don't add dangling symlinks
            if not os.path.exists(file):
                pass
            elif os.path.isdir(file):
                #new_content[file] = get_target(file, is_relative, strip_path)
                filetree = os.walk(file)
                for branch in filetree:
                    path = branch[0]
                    for leaf in branch[2]:
                        abspath = os.path.join(path, leaf)
                        target = get_target(abspath, is_relative, strip_path)
                        if target not in self.content:
                            self.contentsize += os.stat(abspath)[6]
                            new_content[target] = abspath
            else:
                target = get_target(file, is_relative, strip_path)
                if target not in self.content:
                    self.contentsize += os.stat(file)[6]
                new_content[target] = file

        # ... and copy new_content back to current content. We need to directly
        # assign the new content to our observable property to trigger
        # notification messages to our observers!
        self.content = new_content.copy()

        # while we're at it, update free space on disc
        stat = os.statvfs(tmpdir)
        self.discfree = (long(stat[statvfs.F_BAVAIL]) * long(stat[statvfs.F_BSIZE]))

        pass

    def run_cdrecord(self, command):
        process = Popen3(command.split(), 1, 10)
        # Non-blocking, please!
        fcntl(process.fromchild.fileno(), F_SETFL, os.O_NONBLOCK)
        fcntl(process.tochild.fileno(),   F_SETFL, os.O_NONBLOCK)
        fcntl(process.childerr.fileno(),  F_SETFL, os.O_NONBLOCK)

        progress_pattern = 'Track \d+:\s+(\d+) of\s+(\d+) MB written'
        progress_re = re.compile(progress_pattern)
        # parse input from process while it is running
        while 1:
            (r, w, e) = select([process.fromchild, ], [], [process.childerr, ], 1)
            for i in r:
                print i
                proc_output = process.fromchild.read()
                if proc_output:
                    print proc_output
                    if re.search(progress_pattern, proc_output):
                        self.progress = progress_re.search(proc_output).groups()
            while gtk.events_pending():
                gtk.main_iteration()
                print '.',
            self.retval = process.poll()
            if self.retval != -1: break 



# vim: expandtab ts=4
